#!/usr/bin/python3
# -*- coding: utf-8 -*-

import sys
import os
import re
import json

# Определяем тип тестирующей системы
# В отладочных целях можно заменить эти переменные на константы True/False
POLYGON = ("polygon" in os.environ.get("APPDATA", "0").lower()) or ("codeforces" in os.environ.get("APPDATA", "0").lower())
EJUDGE = "EJUDGE" in os.environ
SIRIUS = not(POLYGON or EJUDGE)

# Коды возврата для передачи в функцию quit
OK = 0
WA = 1
PT = 1
PE = 2
CF = 3

# Класс для работы с ответом участника в тестирующей системе Сириуса
class AnswerSMT():
    def __init__(self, userAnswer=None, score=0, maxScore=100):
        '''считывание ответа'''
        if userAnswer is None:
            self.userAnswer = json.loads("".join(sys.stdin.readlines()))['solution']
        else:
            self.userAnswer = json.loads(userAnswer)
        
        
        '''считаем количество элементов ввода и делаем список ответов по элементам ввода'''
        if type(self.userAnswer) is dict:
            self.elements = 1
            if type(self.userAnswer['solution']) is dict:
                self.userAnswer = [[self.userAnswer['solution']['message']]]
            elif type(self.userAnswer['solution']) is int:
                self.userAnswer = [[self.userAnswer['solution']]]
            else:
                self.userAnswer = [self.userAnswer['solution']]
        else: 
            self.elements = len(self.userAnswer)
            a = []
            for i in range(0, self.elements):
                if type(self.userAnswer[i]['solution']) is int:
                    a.append([self.userAnswer[i]['solution']])
                else:
                    a.append(self.userAnswer[i]['solution'])
            self.userAnswer = a
        
        '''считаем, сколько полей ввода для каждого элемента ввода'''
        self.userAnswerLength = []
        for i in range(0, self.elements):
            if type(self.userAnswer[i]) is list:
                self.userAnswerLength.append(len(self.userAnswer[i]))
            elif self.userAnswer[i] != '':
                self.userAnswerLength.append(1)
            else:
                self.userAnswerLength.append(0)
        
        self.comment = ''
        self.score = score
        self.maxScore = maxScore
        self.verdict = 'WA'
          
    def answer(self, element=0, field=0):
        '''Возвращает ответ участника из элмента ввода с номером element.
        Если в элементе ввода несколько полей ввода, то можно указать номер поля ввода'''
        if element >= self.elements:
            return None
        else:
            a = self.userAnswer[element]
        
        if a is None:
            return None
        if field >= len(a):
            return None
        else: 
            return None if a[field] is None else str(a[field])
    
    def fields_in_element(self, element=0):
        '''Возвращает информацию о количестве полей ввода в элементе ввода'''
        if len(self.userAnswerLength) <= element: 
            return 0
        else:
            return self.userAnswerLength[element]
    
    def print_answer(self):
        '''посмотреть всё содержимое для отладки'''
        print("user answer:")
        print(self.userAnswer)
        print("number of elements")
        print(self.elements)
        print('length of elements')
        print(self.userAnswerLength)
    
    def quit(self, comment=None, score=None, maxScore=None, verdict=None):
        if not comment is None:
            self.comment = comment
        
        if not score is None:
            self.score = score
        
        if not maxScore is None:
            self.maxScore = maxScore
            
        if not verdict is None:
            self.verdict = verdict
            
        print(json.dumps({"maxScore":self.maxScore,
                          "score":self.score,
                          "comment":self.comment,
                          "verdict":self.verdict}))
        sys.exit(0)


def quit(status, log="", score=0):
    if SIRIUS:
        if status in [OK, WA] and 0 < score < 100:
            status = PT
        comment = log.replace("\n", "\n\n") # Дублируем концы строк для разметки
        sirius_ans_obj.quit(verdict=status, comment=comment, score=score)
        return
    
    if POLYGON: # Нужно в Polygon, т.к. там Windows
        sys.stdout.reconfigure(encoding='utf-8')
        sys.stderr.reconfigure(encoding='utf-8')

    # Выводим набранные баллы
    if status in [OK, WA]:
        if POLYGON:
            sys.stderr.write("points " + str(score) + "\n")
            status = 7 # Код возврата полигона, сообщающий о наличии баллов
        elif EJUDGE:
            sys.stdout.write(str(score) + "\n")
    
    # Выводим протокол проверки
    sys.stderr.write(log)
    
    # Возвращаем код возврата
    sys.exit(status)


def init():
    global OK, WA, PE, CF, PT
    global sirius_ans_obj
    if SIRIUS:
        PE = "PE"
        WA = "WA"
        OK = "OK"
        СF = "CF"
        PT = "PT"
        # При создании объекта можно в конструктор в целях отладки передать строку с JSON, тогда ответ будет считан с этого JSON, иначе из stdin
        sirius_ans_obj = AnswerSMT()


init()

########################################################################################
# Код дальше можно править

N_ans = 5

# Код этой функции надо исправить так, чтобы она возвращала ответ, полученный из JSON в Сириусе
# или полученный из файла в тестирующих системах Ejudge или Polygon
# Формат возвращаемого значения - произвольный, например, одна строка или список строк
# При необходимости можно прямо в этом коде делать quit(PE, ...)
def get_answer():
    ans = []
    if SIRIUS:
        for i in range(N_ans):
            elem = sirius_ans_obj.answer(element=i)
            if elem is None:
                elem = ""
            ans.append(elem.strip())
    else:
        fin = open(sys.argv[2], "r", encoding="utf8")
        for elem in fin:
            ans.append(elem.strip())
        while len(ans) < N_ans:
            ans.append("")
    return ans

ans = get_answer()

for i in range(len(ans)):
    ans[i] = ans[i].strip()


def solve(n):
    n = sorted([c for c in n if c.isdigit()])
    ans = -1
    m = len(n)
    for mask in range(1, 2 ** m):
        s = ""
        for i in range(m):
            if mask & (1 << i):
                s += n[i]
        s = int(s)
        if s % 9 == 0 and (s < ans or ans == -1):
            ans = s
    return str(ans)

TASKS = ["1, 3, 5, 6, 7", "7, 3, 1, 2, 6, 5", "2, 2, 4, 3, 3, 3, 1, 1, 1, 1", "6, 6, 8, 4, 4, 2, 2, 4, 6, 2", "4, 8, 4, 8, 4, 8, 4, 8, 4, 8, 4, 8"]

score = 0
LOG = ""

def check(s, t):
    for c in s:
        i = t.find(c)
        if i == -1:
            return False
        t = t[:i] + t[i+1:]
    return True


def only_digits(s):
    res = ""
    for c in s:
        if c.isdigit():
            res += c
    return res


for i in range(N_ans):
    LOG += "\n" + ["Первое", "Второе", "Третье", "Четвёртое", "Пятое"][i] + " задание\n"
    ans_num = only_digits(ans[i])
    if not ans_num:
        LOG += "Ответ не дан (в ответе нет ни одной цифры).\n"
        continue

    LOG += f"Ваш ответ: {ans[i]}\n"
    if not check(ans_num, TASKS[i]):
        LOG += f"Это число нельзя составить из набора цифр \u00AB{TASKS[i]}\u00BB\n"
        continue
    if int(ans_num) % 9 != 0:
        LOG += "Это число не делится на 9.\n"
        continue
    corr_ans = solve(TASKS[i])
    if ans_num == corr_ans:
        LOG += "Это правильный ответ. За него даётся 20 баллов.\n"
        score += 20
        continue
    extra_len = len(ans_num) - len(corr_ans)
    ordered = list(ans_num) == sorted(list(ans_num))
    LOG += f"Это не лучший ответ, правильный ответ {corr_ans}. "
    if   extra_len == 0 and ordered:
        s = 16
    elif extra_len == 0 and not ordered:
        s = 15
    elif extra_len == 1 and ordered:
        s = 10
    elif extra_len == 1 and not ordered:
        s = 8
    elif extra_len == 2 and ordered:
        s = 6
    elif extra_len == 2 and not ordered:
        s = 5
    elif ordered:
        s = 4
    else:
        s = 3
    LOG += f"За этот ответ вы получаете {s} баллов.\n"
    score += s
    
LOG += f"Суммарный балл: {score}.\n"
quit(OK if score == 100 else WA, LOG, score=score)

