#!/usr/bin/python3
# -*- coding: utf-8 -*-

import sys
import os
import re
import json

# Определяем тип тестирующей системы
# В отладочных целях можно заменить эти переменные на константы True/False
POLYGON = ("polygon" in os.environ.get("APPDATA", "0").lower()) or ("codeforces" in os.environ.get("APPDATA", "0").lower())
EJUDGE = "EJUDGE" in os.environ
SIRIUS = not(POLYGON or EJUDGE)

# Коды возврата для передачи в функцию quit
OK = 0
WA = 1
PT = 1
PE = 2
CF = 3

# Класс для работы с ответом участника в тестирующей системе Сириуса
class AnswerSMT():
    def __init__(self, userAnswer=None, score=0, maxScore=100):
        '''считывание ответа'''
        if userAnswer is None:
            self.userAnswer = json.loads("".join(sys.stdin.readlines()))['solution']
        else:
            self.userAnswer = json.loads(userAnswer)
        
        
        '''считаем количество элементов ввода и делаем список ответов по элементам ввода'''
        if type(self.userAnswer) is dict:
            self.elements = 1
            if type(self.userAnswer['solution']) is dict:
                self.userAnswer = [[self.userAnswer['solution']['message']]]
            elif type(self.userAnswer['solution']) is int:
                self.userAnswer = [[self.userAnswer['solution']]]
            else:
                self.userAnswer = [self.userAnswer['solution']]
        else: 
            self.elements = len(self.userAnswer)
            a = []
            for i in range(0, self.elements):
                if type(self.userAnswer[i]['solution']) is int:
                    a.append([self.userAnswer[i]['solution']])
                else:
                    a.append(self.userAnswer[i]['solution'])
            self.userAnswer = a
        
        '''считаем, сколько полей ввода для каждого элемента ввода'''
        self.userAnswerLength = []
        for i in range(0, self.elements):
            if type(self.userAnswer[i]) is list:
                self.userAnswerLength.append(len(self.userAnswer[i]))
            elif self.userAnswer[i] != '':
                self.userAnswerLength.append(1)
            else:
                self.userAnswerLength.append(0)
        
        self.comment = ''
        self.score = score
        self.maxScore = maxScore
        self.verdict = 'WA'
          
    def answer(self, element=0, field=0):
        '''Возвращает ответ участника из элмента ввода с номером element.
        Если в элементе ввода несколько полей ввода, то можно указать номер поля ввода'''
        if element >= self.elements:
            return None
        else:
            a = self.userAnswer[element]
        
        if a is None:
            return None
        if field >= len(a):
            return None
        else: 
            return None if a[field] is None else str(a[field])
    
    def fields_in_element(self, element=0):
        '''Возвращает информацию о количестве полей ввода в элементе ввода'''
        if len(self.userAnswerLength) <= element: 
            return 0
        else:
            return self.userAnswerLength[element]
    
    def print_answer(self):
        '''посмотреть всё содержимое для отладки'''
        print("user answer:")
        print(self.userAnswer)
        print("number of elements")
        print(self.elements)
        print('length of elements')
        print(self.userAnswerLength)
    
    def quit(self, comment=None, score=None, maxScore=None, verdict=None):
        if not comment is None:
            self.comment = comment
        
        if not score is None:
            self.score = score
        
        if not maxScore is None:
            self.maxScore = maxScore
            
        if not verdict is None:
            self.verdict = verdict
            
        print(json.dumps({"maxScore":self.maxScore,
                          "score":self.score,
                          "comment":self.comment,
                          "verdict":self.verdict}))
        sys.exit(0)


def quit(status, log="", score=0):
    if SIRIUS:
        if status in [OK, WA] and 0 < score < 100:
            status = PT
        comment = log.replace("\n", "\n\n") # Дублируем концы строк для разметки
        sirius_ans_obj.quit(verdict=status, comment=comment, score=score)
        return
    
    if POLYGON: # Нужно в Polygon, т.к. там Windows
        sys.stdout.reconfigure(encoding='utf-8')
        sys.stderr.reconfigure(encoding='utf-8')

    # Выводим набранные баллы
    if status in [OK, WA]:
        if POLYGON:
            sys.stderr.write("points " + str(score) + "\n")
            status = 7 # Код возврата полигона, сообщающий о наличии баллов
        elif EJUDGE:
            sys.stdout.write(str(score) + "\n")
    
    # Выводим протокол проверки
    sys.stderr.write(log)
    
    # Возвращаем код возврата
    sys.exit(status)


def init():
    global OK, WA, PE, CF, PT
    global sirius_ans_obj
    if SIRIUS:
        PE = "PE"
        WA = "WA"
        OK = "OK"
        СF = "CF"
        PT = "PT"
        # При создании объекта можно в конструктор в целях отладки передать строку с JSON, тогда ответ будет считан с этого JSON, иначе из stdin
        sirius_ans_obj = AnswerSMT()


init()

########################################################################################
# Код дальше можно править

# Код этой функции надо исправить так, чтобы она возвращала ответ, полученный из JSON в Сириусе
# или полученный из файла в тестирующих системах Ejudge или Polygon
# Формат возвращаемого значения - произвольный, например, одна строка или список строк
# При необходимости можно прямо в этом коде делать quit(PE, ...)
def get_answer():
    if SIRIUS:
        ans = sirius_ans_obj.answer(element=0, field=0)
        if ans is None:
            ans = ""
    else:
        fin = open(sys.argv[2], "r", encoding="utf8")
        ans = fin.readline().rstrip()
    return ans

ans = get_answer().lower().strip()

ans = ans.replace("а", "a") # Русская на английскую
ans = ans.replace("б", "b") # Русская на английскую
ans = ans.replace("в", "b") # Русская на английскую
ans = ans.replace("с", "c") # Русская на английскую

ans1 = ""
for c in ans:
    if c.isalpha():
        if c in "abc":
            ans1 += c
        else:
            quit(PE, f"Ответ содержит недопустимый символ \u00AB{c}\u00BB")

LOG = ""
LOG += "Сданный ответ: " + ans1 + "\n\n"

VISITED_EDGES = set()
ALL_EDGES = {
    ((0,0,0), (0,0,1)),
    ((0,0,0), (0,1,0)),
    ((0,0,0), (1,0,0)),
    ((0,0,1), (0,1,1)),
    ((0,0,1), (1,0,1)),
    ((0,1,0), (0,1,1)),
    ((0,1,0), (1,1,0)),
    ((1,0,0), (1,0,1)),
    ((1,0,0), (1,1,0)),
    ((0,1,1), (1,1,1)),
    ((1,0,1), (1,1,1)),
    ((1,1,0), (1,1,1)),
}

x, y, z = 0, 0, 0

def coord_to_str(x, y, z):
    x = '0' if x == 0 else 'a'
    y = '0' if y == 0 else 'b'
    z = '0' if z == 0 else 'c'
    return f"({x}, {y}, {z})"

LOG += "Имитация движения муравья\n"
LOG += f"Муравей находится в точке {coord_to_str(x, y, z)}\n"

for c in ans1:
    LOG += f"Обрабатывается команда ответа: {c}.\n"
    x1, y1, z1 = x, y, z
    if c == "a":
        x1 = 1 - x
    elif c == "b":
        y1 = 1 - y
    else:
        z1 = 1 - z
    LOG += f"Муравей находится в точке {coord_to_str(x1, y1, z1)}\n"
    edge = tuple(sorted([(x, y, z), (x1, y1, z1)]))
    VISITED_EDGES.add(edge)
    x, y, z = x1, y1, z1

UNVISITED_EDGES = ALL_EDGES - VISITED_EDGES

if len(UNVISITED_EDGES) == 0:
    LOG += "\nМуравей посетил все рёбра\n"
else:
    LOG += "\nМуравей посетил *не все* рёбра\n"
    edge = UNVISITED_EDGES.pop()
    LOG += f"Непосещённое муравьём ребро: {coord_to_str(*edge[0])} - {coord_to_str(*edge[1])}\n"
    quit(WA, LOG, score=0)

n = len(ans1)
LOG += f"Количество рёбер в пути муравья: {n}\n"
a = ans1.count("a") - 4
b = ans1.count("b") - 4
c = ans1.count("c") - 4
LOG += f"Количество дополнительных рёбер длины a: {a}\n"
LOG += f"Количество дополнительных рёбер длины b: {b}\n"
LOG += f"Количество дополнительных рёбер длины c: {c}\n"

if n == 15 and a == 3:
    LOG += "Это правильный и самый лучший ответ\n"
    quit(OK, LOG, score=100)
else:
    LOG += "Это не самый лучший ответ, есть ответ короче\n"

penalty = a * 1 + b * 2 + c * 3
min_penalty = (n - 12)
max_penalty = (n - 12) * 3

SCORES = {15: 90, 16: 70, 17: 50, 18: 40, 19: 30}

if n >= max(SCORES.keys()):
    quit(WA, LOG, score = min(SCORES.values()))

d = (SCORES[n] - SCORES[n + 1]) * 0.9
d = (penalty - min_penalty) / (max_penalty - min_penalty) * d
score = SCORES[n] - int(d)

quit(WA, LOG, score=score)
