#!/usr/bin/python3
# -*- coding: utf-8 -*-

import sys
import os
import re
import json

# Определяем тип тестирующей системы
# В отладочных целях можно заменить эти переменные на константы True/False
POLYGON = ("polygon" in os.environ.get("APPDATA", "0").lower()) or ("codeforces" in os.environ.get("APPDATA", "0").lower())
EJUDGE = "EJUDGE" in os.environ
SIRIUS = not(POLYGON or EJUDGE)

# Коды возврата для передачи в функцию quit
OK = 0
WA = 1
PT = 1
PE = 2
CF = 3

# Класс для работы с ответом участника в тестирующей системе Сириуса
class AnswerSMT():
    def __init__(self, userAnswer=None, score=0, maxScore=100):
        '''считывание ответа'''
        if userAnswer is None:
            self.userAnswer = json.loads("".join(sys.stdin.readlines()))['solution']
        else:
            self.userAnswer = json.loads(userAnswer)
        
        
        '''считаем количество элементов ввода и делаем список ответов по элементам ввода'''
        if type(self.userAnswer) is dict:
            self.elements = 1
            if type(self.userAnswer['solution']) is dict:
                self.userAnswer = [[self.userAnswer['solution']['message']]]
            elif type(self.userAnswer['solution']) is int:
                self.userAnswer = [[self.userAnswer['solution']]]
            else:
                self.userAnswer = [self.userAnswer['solution']]
        else: 
            self.elements = len(self.userAnswer)
            a = []
            for i in range(0, self.elements):
                if type(self.userAnswer[i]['solution']) is int:
                    a.append([self.userAnswer[i]['solution']])
                else:
                    a.append(self.userAnswer[i]['solution'])
            self.userAnswer = a
        
        '''считаем, сколько полей ввода для каждого элемента ввода'''
        self.userAnswerLength = []
        for i in range(0, self.elements):
            if type(self.userAnswer[i]) is list:
                self.userAnswerLength.append(len(self.userAnswer[i]))
            elif self.userAnswer[i] != '':
                self.userAnswerLength.append(1)
            else:
                self.userAnswerLength.append(0)
        
        self.comment = ''
        self.score = score
        self.maxScore = maxScore
        self.verdict = 'WA'
          
    def answer(self, element=0, field=0):
        '''Возвращает ответ участника из элмента ввода с номером element.
        Если в элементе ввода несколько полей ввода, то можно указать номер поля ввода'''
        if element >= self.elements:
            return None
        else:
            a = self.userAnswer[element]
        
        if a is None:
            return None
        if field >= len(a):
            return None
        else: 
            return None if a[field] is None else str(a[field])
    
    def fields_in_element(self, element=0):
        '''Возвращает информацию о количестве полей ввода в элементе ввода'''
        if len(self.userAnswerLength) <= element: 
            return 0
        else:
            return self.userAnswerLength[element]
    
    def print_answer(self):
        '''посмотреть всё содержимое для отладки'''
        print("user answer:")
        print(self.userAnswer)
        print("number of elements")
        print(self.elements)
        print('length of elements')
        print(self.userAnswerLength)
    
    def quit(self, comment=None, score=None, maxScore=None, verdict=None):
        if not comment is None:
            self.comment = comment
        
        if not score is None:
            self.score = score
        
        if not maxScore is None:
            self.maxScore = maxScore
            
        if not verdict is None:
            self.verdict = verdict
            
        print(json.dumps({"maxScore":self.maxScore,
                          "score":self.score,
                          "comment":self.comment,
                          "verdict":self.verdict}))
        sys.exit(0)


def quit(status, log="", score=0):
    if SIRIUS:
        if status in [OK, WA] and 0 < score < 100:
            status = PT
        comment = log.replace("\n", "\n\n") # Дублируем концы строк для разметки
        sirius_ans_obj.quit(verdict=status, comment=comment, score=score)
        return
    
    if POLYGON: # Нужно в Polygon, т.к. там Windows
        sys.stdout.reconfigure(encoding='utf-8')
        sys.stderr.reconfigure(encoding='utf-8')

    # Выводим набранные баллы
    if status in [OK, WA]:
        if POLYGON:
            sys.stderr.write("points " + str(score) + "\n")
            status = 7 # Код возврата полигона, сообщающий о наличии баллов
        elif EJUDGE:
            sys.stdout.write(str(score) + "\n")
    
    # Выводим протокол проверки
    sys.stderr.write(log)
    
    # Возвращаем код возврата
    sys.exit(status)


def init():
    global OK, WA, PE, CF, PT
    global sirius_ans_obj
    if SIRIUS:
        PE = "PE"
        WA = "WA"
        OK = "OK"
        СF = "CF"
        PT = "PT"
        # При создании объекта можно в конструктор в целях отладки передать строку с JSON, тогда ответ будет считан с этого JSON, иначе из stdin
        sirius_ans_obj = AnswerSMT()


init()

########################################################################################
# Код дальше можно править

# Код этой функции надо исправить так, чтобы она возвращала ответ, полученный из JSON в Сириусе
# или полученный из файла в тестирующих системах Ejudge или Polygon
# Формат возвращаемого значения - произвольный, например, одна строка или список строк
# При необходимости можно прямо в этом коде делать quit(PE, ...)
def get_answer():
    ans = []
    if SIRIUS:
        for i in range(sirius_ans_obj.fields_in_element()):
            elem = sirius_ans_obj.answer(field=i)
            if elem is not None and elem.strip():
                ans.append(elem.strip())
    else:
        fin = open(sys.argv[2], "r", encoding="utf8")
        for elem in fin:
            if elem.strip():
                ans.append(elem.strip())
    return ans


def extract_t_x(s):
    s_saved = s
    s = s.replace("`", " ")
    s = s.replace("'", " ")
    s = s.replace('"', " ")
    s = s.replace('<', " ")
    s = s.replace('>', " ")
    s = s.replace(chr(0xAB), " ")
    s = s.replace(chr(0xBB), " ")
    s = s.replace(chr(0xAB), " ")
    s = s.replace(',', " ")
    s = s.replace(';', " ")
    s = s.replace(':', " ")
    for c in s:
        if c not in " 0123456789":
            quit(PE, f"Поле ответа \u00AB{s_saved}\u00BB содержит недопустимый символ \u00AB{c}\u00BB\n")
    s = s.split()
    if len(s) != 2:
        quit(PE, f"Поле ответа \u00AB{s_saved}\u00BB неверное, в каждом поле должно быть два числа через запятую\n")
    return (int(s[0]), int(s[1]))


ALL_TIMES = set()
ans = get_answer()
for i in range(len(ans)):
    ans[i] = extract_t_x(ans[i])
    if ans[i][0] in ALL_TIMES:
        quit(PE, f"В ответе в момент времени {ans[i][0]} выпушено несколько торпед. Все времена выстрелов должны быть различны.\n")
    ALL_TIMES.add(ans[i][0])

RIGHT = {10, 12, 20, 32, 42}
LEFT = {18, 26, 28, 30, 48}

n = len(ans)
if n // 10 == 1 or n % 10 in [0, 5, 6, 7, 8, 9]:
    word = "выстрелов"
elif n % 10 in [2, 3, 4]:
    word = "выстрела"
else:
    word = "выстрел"

LOG = f"В вашем ответе {len(ans)} {word}\n\n"

count = 1
for t, x in ans:
    LOG += f"Выстрел \u2116{count}, время {t}, координата {x}\n"
    count += 1
    hit = False
    if x - t in RIGHT:
        LOG += f"Подбит корабль, двигающийся вправо, с начальной координатой {x - t}\n"
        RIGHT.remove(x - t)
        hit = True
    if x + t in LEFT:
        LOG += f"Подбит корабль, двигающийся влево, с начальной координатой {x + t}\n"
        LEFT.remove(x + t)
        hit = True
    if not hit:
        LOG += "Этот выстрел не подбил ни один корабль\n"

    LOG += "\n"

if LEFT | RIGHT:
    LOG += "Не все корабли были подбиты\n"
    LOG += "Начальные координаты неподбитых кораблей: " + str(sorted(LEFT | RIGHT))[1:-1] + "\n"
    quit(WA, LOG, score=0)
else:
    LOG += "Все корабли были подбиты\n"

if len(ans) == 6:
    LOG += "Это лучший ответ\n"
    quit(OK, LOG, score=100)
else:
    LOG += "Но это не самый лучший ответ, есть ответ использующий меньше выстрелов\n"

if len(ans) == 7:
    quit(OK, LOG, score=60)
elif len(ans) == 8:
    quit(OK, LOG, score=40)
elif len(ans) == 9:
    quit(OK, LOG, score=30)
elif len(ans) == 10:
    quit(OK, LOG, score=20)
else:
    quit(OK, LOG, score=10)

