#!/usr/bin/python3
# -*- coding: utf-8 -*-

import sys
import os
import re
import json

# Определяем тип тестирующей системы
# В отладочных целях можно заменить эти переменные на константы True/False
POLYGON = ("polygon" in os.environ.get("APPDATA", "0").lower()) or ("codeforces" in os.environ.get("APPDATA", "0").lower())
EJUDGE = "EJUDGE" in os.environ
SIRIUS = not(POLYGON or EJUDGE)

# Коды возврата для передачи в функцию quit
OK = 0
WA = 1
PT = 1
PE = 2
CF = 3

# Класс для работы с ответом участника в тестирующей системе Сириуса
class AnswerSMT():
    def __init__(self, userAnswer=None, score=0, maxScore=100):
        '''считывание ответа'''
        if userAnswer is None:
            self.userAnswer = json.loads("".join(sys.stdin.readlines()))['solution']
        else:
            self.userAnswer = json.loads(userAnswer)
        
        
        '''считаем количество элементов ввода и делаем список ответов по элементам ввода'''
        if type(self.userAnswer) is dict:
            self.elements = 1
            if type(self.userAnswer['solution']) is dict:
                self.userAnswer = [[self.userAnswer['solution']['message']]]
            elif type(self.userAnswer['solution']) is int:
                self.userAnswer = [[self.userAnswer['solution']]]
            else:
                self.userAnswer = [self.userAnswer['solution']]
        else: 
            self.elements = len(self.userAnswer)
            a = []
            for i in range(0, self.elements):
                if type(self.userAnswer[i]['solution']) is int:
                    a.append([self.userAnswer[i]['solution']])
                else:
                    a.append(self.userAnswer[i]['solution'])
            self.userAnswer = a
        
        '''считаем, сколько полей ввода для каждого элемента ввода'''
        self.userAnswerLength = []
        for i in range(0, self.elements):
            if type(self.userAnswer[i]) is list:
                self.userAnswerLength.append(len(self.userAnswer[i]))
            elif self.userAnswer[i] != '':
                self.userAnswerLength.append(1)
            else:
                self.userAnswerLength.append(0)
        
        self.comment = ''
        self.score = score
        self.maxScore = maxScore
        self.verdict = 'WA'
          
    def answer(self, element=0, field=0):
        '''Возвращает ответ участника из элмента ввода с номером element.
        Если в элементе ввода несколько полей ввода, то можно указать номер поля ввода'''
        if element >= self.elements:
            return None
        else:
            a = self.userAnswer[element]
        
        if a is None:
            return None
        if field >= len(a):
            return None
        else: 
            return None if a[field] is None else str(a[field])
    
    def fields_in_element(self, element=0):
        '''Возвращает информацию о количестве полей ввода в элементе ввода'''
        if len(self.userAnswerLength) <= element: 
            return 0
        else:
            return self.userAnswerLength[element]
    
    def print_answer(self):
        '''посмотреть всё содержимое для отладки'''
        print("user answer:")
        print(self.userAnswer)
        print("number of elements")
        print(self.elements)
        print('length of elements')
        print(self.userAnswerLength)
    
    def quit(self, comment=None, score=None, maxScore=None, verdict=None):
        if not comment is None:
            self.comment = comment
        
        if not score is None:
            self.score = score
        
        if not maxScore is None:
            self.maxScore = maxScore
            
        if not verdict is None:
            self.verdict = verdict
            
        print(json.dumps({"maxScore":self.maxScore,
                          "score":self.score,
                          "comment":self.comment,
                          "verdict":self.verdict}))
        sys.exit(0)


def quit(status, log="", score=0):
    if SIRIUS:
        if status in [OK, WA] and 0 < score < 100:
            status = PT
        comment = log.replace("\n", "\n\n") # Дублируем концы строк для разметки
        sirius_ans_obj.quit(verdict=status, comment=comment, score=score)
        return
    
    if POLYGON: # Нужно в Polygon, т.к. там Windows
        sys.stdout.reconfigure(encoding='utf-8')
        sys.stderr.reconfigure(encoding='utf-8')

    # Выводим набранные баллы
    if status in [OK, WA]:
        if POLYGON:
            sys.stderr.write("points " + str(score) + "\n")
            status = 7 # Код возврата полигона, сообщающий о наличии баллов
        elif EJUDGE:
            sys.stdout.write(str(score) + "\n")
    
    # Выводим протокол проверки
    sys.stderr.write(log)
    
    # Возвращаем код возврата
    sys.exit(status)


def init():
    global OK, WA, PE, CF, PT
    global sirius_ans_obj
    if SIRIUS:
        PE = "PE"
        WA = "WA"
        OK = "OK"
        СF = "CF"
        PT = "PT"
        # При создании объекта можно в конструктор в целях отладки передать строку с JSON, тогда ответ будет считан с этого JSON, иначе из stdin
        sirius_ans_obj = AnswerSMT()


init()

########################################################################################

DATA = """
2
3
-1
4
4
1
2
1
0
5
-3
-1
-2
4
-4
-4
-2
4
5
3
-1
0
2
-5
0
-2
-2
1
-1
-2
-3
0
3
-2
0
-2
3
-2
1
5
3
4
-5
3
-1
0
-3
-2
1
2
-2
2
2
-1
-4
4
2
-4
4
3
2
-1
1
-3
-2
-1
4
-4
-1
-2
0
4
-3
4
-3
-1
-3
3
3
2
2
-2
-5
4
-3
0
-3
4
-3
-2
-2
-2
4
-1
-3
-1
-2
-2
-3
-3
0
3
2
4
4
4
-3
4
-4
5
2
3
4
-3
-3
-3
-1
-1
2
1
2
-3
-4
-3
-1
-3
-1
2
5
-2
4
5
-4
0
-3
5
-1
-3
3
-2
0
-3
5
0
-1
-3
1
-1
4
0
4
-2
0
-1
4
-5
1
-4
0
4
0
5
3
-2
-5
0
-2
3
4
0
2
-1
-1
-4
-3
-1
3
2
3
4
-5
-4
1
-3
2
-2
2
0
1
3
-1
-3
-2
0
3
-2
0
1
3
-4
5
0
2
-5
-3
0
4
-5
-2
4
2
-5
5
-4
2
5
3
-5
1
-5
-2
5
-2
-3
5
-3
-4
1
4
0
3
3
1
1
-5
2
-1
-3
5
-3
-5
1
-4
-1
-2
1
2
1
-4
5
1
-2
3
5
-3
-3
-3
-3
5
4
-5
-3
4
-1
-4
-2
-2
1
0
4
-2
1
-1
-3
0
3
-2
-3
0
-2
2
-1
-1
-1
-1
4
2
5
-1
4
0
4
1
2
2
-5
1
4
1
-2
4
1
2
-1
2
-4
-4
-4
4
-3
2
-1
4
0
-5
-3
1
3
-3
-3
1
3
-4
0
-1
-3
0
-1
-3
-1
4
1
-3
1
1
3
1
1
0
4
4
1
0
1
-2
2
-4
4
-2
1
3
-4
1
2
-3
1
4
0
-4
-3
-2
0
2
-5
-4
2
5
-3
4
1
1
-5
-4
5
-3
-3
-2
1
-1
3
-1
-2
0
3
4
-2
-3
4
2
-3
0
3
1
-3
2
-4
-2
0
0
1
2
2
-2
5
-1
5
-1
3
1
-5
1
-5
-2
2
5
2
-1
-2
2
3
3
-2
1
-1
2
-5
3
-2
-5
3
1
4
-5
3
1
-2
-1
-2
-3
3
3
0
3
-1
-1
-5
-3
2
-1
-4
4
-1
2
4
-3
-4
1
-3
-2
4
4
-3
2
4
4
-4
0
1
-5
-3
-1
3
-1
3
-1
2
1
-4
-2
4
-4
-2
5
-1
-2
1
0
5
3
1
-3
1
-4
3
-2
-4
-4
-3
4
2
1
5
4
5
-1
-4
-2
5
1
1
3
5
-3
-5
-4
1
-1
2
-2
5
5
4
-2
1
-1
3
1
-1
-2
3
-3
1
2
-2
-2
3
-2
-5
5
2
3
-1
0
2
-4
-1
-1
3
2
-3
-1
2
2
3
0
0
-3
3
1
-1
3
-2
3
2
-3
0
-1
3
-4
3
-2
-4
-1
2
-4
-5
1
-5
2
2
2
-1
-4
-2
-2
4
-3
-2
2
-2
-3
2
-3
5
-4
-1
5
1
-2
-3
5
-4
-2
5
2
-3
4
-1
1
2
-4
-3
-3
-3
0
5
4
5
-3
2
-1
3
1
1
-5
5
2
0
-2
3
1
-5
-1
1
1
-1
-1
4
-2
3
-4
0
3
2
0
0
2
-1
-4
-4
1
-4
5
-1
2
-4
-1
2
0
-4
-2
2
-4
2
5
-5
1
3
5
-2
-4
4
-5
3
-4
4
-5
-1
-3
-3
5
1
5
5
4
-3
4
3
2
3
-3
-1
3
-2
3
-4
-1
3
-4
-5
-1
-3
-2
-5
1
2
-4
-2
-1
-3
-3
0
-3
0
2
-2
0
2
4
0
2
4
2
-2
3
-2
-4
-2
5
2
4
2
-1
-1
2
-1
4
0
2
2
2
-2
2
1
-1
3
0
3
-5
-1
-5
-2
-3
-2
4
0
1
3
-3
4
2
0
1
1
1
1
-2
2
-1
-3
-1
3
0
1
2
3
1
2
-4
-5
-4
0
-4
1
-3
4
3
-5
2
2
-2
0
2
-1
-3
4
-3
-5
0
3
3
-1
0
3
1
1
-3
-1
0
-4
4
-1
2
-1
1
-4
1
-5
2
-1
1
-1
1
0
0
1
1
1
-5
-2
2
-4
3
5
0
4
-5
4
0
1
2
-4
-2
4
-5
3
0
-1
-4
1
-1
4
-4
4
3
2
-5
0
-1
2
-5
-5
2
2
-2
1
1
-3
-4
3
-4
-2
4
5
-1
1
-4
1
3
-1
0
-2
2
-4
2
3
-2
-1
-1
4
4
-4
1
4
-4
0
2
-4
3
-1
3
3
-5
1
-4
2
5
2
-5
1
2
2
-5
-2
1
5
-3
2
-3
-1
-2
0
-4
1
4
-1
-4
2
-1
-2
3
1
-3
0
2
5
4
1
1
0
-3
-4
2
-3
0
5
4
-3
1
0
-5
1
0
-4
-2
-2
5
4
4
-2
2
-2
1
3
-1
-4
-4
5
2
-2
-3
-1
-4
-2
5
1
4
-3
0
-1
1
-5
3
-5
-3
-3
11
5
0
-3
3
5
3
0
-3
5
-4
5
4
-2
"""

DATA = list(map(int, DATA.split()))

# Код дальше можно править

N_ans = 3

# Код этой функции надо исправить так, чтобы она возвращала ответ, полученный из JSON в Сириусе
# или полученный из файла в тестирующих системах Ejudge или Polygon
# Формат возвращаемого значения - произвольный, например, одна строка или список строк
# При необходимости можно прямо в этом коде делать quit(PE, ...)
def get_answer():
    ans = []
    if SIRIUS:
        for i in range(N_ans):
            elem = sirius_ans_obj.answer(field=i)
            if elem is None:
                elem = ""
            ans.append(elem.strip())
    else:
        fin = open(sys.argv[2], "r", encoding="utf8")
        for elem in fin:
            ans.append(elem.strip())
        while len(ans) < N_ans:
            ans.append()
    return ans

ans = get_answer()

for i in range(len(ans)):
    ans[i] = ans[i].strip()
    if not ans[i].isdigit():
        quit(PE, "В ответе должны быть три натуральных числа\n")
    ans[i] = int(ans[i])

if sum(ans) != 1000:
    quit(PE, f"Сумма чисел в ответе должна быть равна 1000, сумма чисел в вашем ответе равна {sum(ans)}\n")

a, b, c = ans
s1 = sum(DATA[:a])
s2 = sum(DATA[a:a+b])
s3 = sum(DATA[a+b:a+b+c])

if s1 != s2 or s2 != s3:
    quit(WA, f"Суммы уровней поддержки на трибунах в вашем ответе равны {s1}, {s2}, {s3}. Эти числа неравны.\n", score=0)

d = max(a, b, c) - min(a, b, c)

def calc_score(x):
    m1 = 29
    m2 = 965
    x = (x - m1) / (m2 - m1)
    x = x ** (1/3)
    x = 1 - x
    x = 40 + x * 60
    return x

if d == 29:
    quit(OK, "Это лучший ответ\n", score=100)
else:
    quit(WA, f"Этот ответ правильный, но не лучший. В вашем ответе разница между самой большой и самой маленькой трибуной равна {d}, а в лучшем ответе разность равна 29.\n", score=calc_score(d))

